%% Buntabgleich zwischen zwei Bildern
%
% Angleich eines Farbtons im Hintergrund eines Bildes an den Farbton im 
% Hintergrund eines anderen Bildes. 
% Beide Farben muessen ausreichend aehnlich sein und ausreichend Saettigung 
% aufweisen.
%
% 14.01.2018 Nicolas

clear all
close all

%% Einstellungen/Dateien
% Src: Farbquelle
% Dst: Umzurechnendes Bild

% Dateien 
Src.filename = 'Usb-Isolator02_400.jpg';
%Dst.filename = 'Powerpole-Verteiler_offen_400.jpg'; % Einfaches Beispiel
Dst.filename = 'hf-sniffer_02.jpg';                 % Normales Beispiel
%Dst.filename = 'hochleistungssirene01_400.jpg';     % Fatales Beispiel


% Reihenfolge der Umrechnung (meanrgb|meanhsv|donothing)
%  meanrgb: Mittelwertbildung im RGB-Modus
%  meanhsv: Mittelwertbildung im HSV-Modus
%mode = 'meanrgb';
mode = 'meanhsv';
%mode = 'donothing';



Src.location = 'northeast'; % Anordnung des Quell-Blaus
Src.diaPx = 150;            % Durchmesser der Quell-Blau-Region (Quadrat)
Dst.location = 'southwest'; % Anordnung des Ziel-Blaus
Dst.diaPx = 100;            % Durchmesser der Ziel-Blau-Region (Quadrat)

suffix = '_cnv';


%% Bildverarbeitung

% Bilder einlesen
Src.img = imread(Src.filename);
Dst.img = imread(Dst.filename);
Src.size = size(Src.img);
Dst.size = size(Dst.img);

% Maske fuer Sample-Area bestimmen
Src.samplemask = calcmask(Src.size, Src.location, Src.diaPx, 'circle');
Dst.samplemask = calcmask(Dst.size, Dst.location, Dst.diaPx, 'circle');

% Maske fuer Darstellung der Sample-Area bestimmten
Src.dispmask = calcmask(Src.size, Src.location, Src.diaPx, 'ring');
Dst.dispmask = calcmask(Dst.size, Dst.location, Dst.diaPx, 'ring');


% Quellbilder darstellen: Bilder und ausgewaehlter Bereich
figure
A = Src.img;
A(Src.dispmask) = 0;
imshow(A);
title('Src image - Sample area');

figure
A = Dst.img;
A(Dst.dispmask) = 0;
imshow(A)
title('Dst image - Sample area');




%% Umrechnungfaktor fuer Bild 2 ermitteln:
% Eepraesentative, gemittelte Farben der Sample-Region von Quell- und 
% Zielbild ermitteln und daraus den Umrechnungsfaktor des Zielbildes bilden

% Alle Pixel aus der Sample-Area einsammeln
nEl1 = sum(sum(sum(Src.samplemask)));
nEl2 = sum(sum(sum(Dst.samplemask)));
assert( rem(nEl1, 3) == 0 && rem(nEl2, 3) == 0  );
SrcArea = zeros(nEl1/3, 3);
DstArea = zeros(nEl2/3, 3);
SrcArea(:) = Src.img(Src.samplemask);
DstArea(:) = Dst.img(Dst.samplemask);


% Gemittelte Farbe im HSV-Format
switch( lower(mode) )
    case 'meanrgb'
        % Erst mitteln, dann HSV-Wert angleichen
        Src.hsv = rgb2hsv(mean(SrcArea));
        Dst.hsv = rgb2hsv(mean(DstArea));
        
    case 'meanhsv'
        Src.hsv = mean(rgb2hsv(SrcArea));        
        Dst.hsv = mean(rgb2hsv(DstArea));
        
    case 'donothing'
        Src.hsv = [1 1 1];
        Dst.hsv = Src.hsv;
        
    otherwise
        error('unknown mode: try ''meanrgb'' or ''meanhsv'' ');
end

% Wenn dreifach indizierte Matrix vorliegt, in einfachen Vektor kollabieren
Src.hsv = Src.hsv(:);
Dst.hsv = Dst.hsv(:);

% Umrechnungsfaktor zwischen den Blautnen
h = Src.hsv./Dst.hsv;



%% Farbe im Zielbild anpassen an Farbe im Quellbild

% Komplettes Bild in HSV umrechnen
DstI = rgb2hsv(Dst.img);

% Angleich der drei HSV-Farbkomponenten
DstI(:,:,1) = DstI(:,:,1)*h(1);
DstI(:,:,2) = DstI(:,:,2)*h(2);
DstI(:,:,3) = DstI(:,:,3)*h(3);

% Zurueck in RGB
DstI = hsv2rgb(DstI);

% Zielbild fuer Darstellung als Kombination aus alten und neuen Farben
% Darstellen
mid = floor(Dst.size(2)/2);
left = Dst.img(:,1:mid,:);                % uint8
right = uint8(DstI(:,(mid+1):end,:)*255); % double
Combi = [left right];

figure
imshow(Combi)
title('Preview')


%% Ziel-Bild schreiben
assert(isa(DstI, 'double'));
[p,n,e] = fileparts(Dst.filename);
dstfile = fullfile(p, [n suffix e]);
fprintf('writing ''%s''\n', dstfile);
imwrite(DstI, dstfile);

