function mask = calcmask(masksize, location, diaPx, mode)
% Logisches Arry mit rechteckiger oder kreisfoermiger Maske erstellen
%  masksize: bildgroesse (Pixel)
%  location: Himmelsrichtung, Ausrichtung im Bild
%  diaPx:    Durchmesser Maskenausschnitt
%  mode:     rect|circle|ring
%
% 13.01.2018 Nicolas



assert( round(diaPx) == diaPx );


%% Region auswaehlen
% Rechteckige Indexmenge anhand der Himmelsrichtung ausrechnen

assert( diaPx <= masksize(1) && diaPx <= masksize(2));


first1  = 1:diaPx;
first2  = first1;
middle1 = masksize(1)/2-diaPx/2+(1:diaPx);
middle2 = masksize(2)/2-diaPx/2+(1:diaPx);
last1   = masksize(1)-diaPx+(1:diaPx);
last2   = masksize(2)-diaPx+(1:diaPx); 

switch lower(location)
    case 'north'
        idx1 = first1;
        idx2 = middle2;
    case 'northeast'
        idx1 = first1;
        idx2 = last2;
    case 'east'
        idx1 = middle1;
        idx2 = last2;
    case 'southeast'
        idx1 = last1;
        idx2 = last2;
    case 'south'
        idx1 = last1;
        idx2 = middle2;             
    case 'southwest'
        idx1 = last1;
        idx2 = first2;
    case 'west'
        idx1 = middle1;
        idx2 = first2;
    case 'northwest'
        idx1 = first1;
        idx2 = first2;
    case 'center'
        idx1 = middle1;
        idx2 = middle2;
    otherwise
        error('region not implemented')

end



mask = false(masksize);


switch( lower(mode) )
    case 'rect'
        % rechteckige Maske
        mask(idx1, idx2, :) = true;

    case 'circle'

        % kreisfoermige Index-Bitmap (etwas rechenaufwendiger)
        idx = true(diaPx, diaPx);
        rs = (diaPx)^2/4;
        c = (diaPx+1)/2*[1 1];
        for ii = 1:diaPx
            for jj = 1:diaPx
                idx(ii,jj) = (ii-c(1))^2 + (jj-c(2))^2 <= rs;
            end
        end
        for ii = 1:size(mask, 3)
            mask(idx1, idx2, ii) = idx;
        end

    case 'ring'

        % ringfoermige Index-Bitmap 
        % TODO: Das ginge mit dem Bresenham-Algorithmus vermutlich 
        % einfacher, besser und schneller
        idx = true(diaPx, diaPx);
        rs = (diaPx)^2/4;
        ri = (diaPx-2)^2/4;
        c = (diaPx+1)/2*[1 1];
        for ii = 1:diaPx
            for jj = 1:diaPx
                pixel = (ii-c(1))^2 + (jj-c(2))^2;
                idx(ii,jj) = pixel <= rs && pixel >= ri;
            end
        end
        for ii = 1:size(mask, 3)
            mask(idx1, idx2, ii) = idx;
        end        
        
    otherwise
        error('unknown mask shape. try ''rect'' or ''circ'' ');
end
